if select(2, UnitClass('player')) ~= 'DRUID' then
	DisableAddOn('Claw')
	return
end

Claw = {}

SLASH_Claw1, SLASH_Claw2 = '/claw', '/clawesome'

local function InitializeVariables()
	for k, v in pairs({ -- defaults
		locked = false,
		scales = {
			main = 1,
			previous = 0.7,
			cooldown = 0.7
		},
		frequency = 0.05,
		latency = 0.2,
		previous = true,
		cooldown = true,
		gcd = true,
		dimmer = true,
		miss_effect = true,
		glow = true,
		boss_only = false,
		mark_of_the_wild = true,
		bear = true,
		cat = true,
		prowl = true,
		tigersfury_berserk = false,
		pot = false,
	}) do
		if Claw[k] == nil then
			Claw[k] = v
		end
	end
end

local events, actionButtons = {}, {}
local me, abilityTimer, currentTime = 0, 0, 0

local clawPanel = CreateFrame('Frame', 'clawPanel', UIParent)
clawPanel:SetPoint('CENTER', 0, -169)
clawPanel:SetFrameStrata('BACKGROUND')
clawPanel:SetSize(64, 64)
clawPanel:Hide()
clawPanel:RegisterForDrag('LeftButton')
clawPanel:SetScript('OnDragStart', clawPanel.StartMoving)
clawPanel:SetScript('OnDragStop', clawPanel.StopMovingOrSizing)
clawPanel:SetMovable(true)
clawPanel.icon = clawPanel:CreateTexture(nil, 'BACKGROUND')
clawPanel.icon:SetAllPoints(clawPanel)
clawPanel.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
clawPanel.border = clawPanel:CreateTexture(nil, 'BORDER')
clawPanel.border:SetAllPoints(clawPanel)
clawPanel.border:SetTexture('Interface\\AddOns\\Claw\\border.blp')
clawPanel.gcd = CreateFrame('Cooldown', nil, clawPanel, 'CooldownFrameTemplate')
clawPanel.gcd:SetAllPoints(clawPanel)
clawPanel.dimmer = clawPanel:CreateTexture(nil, 'OVERLAY')
clawPanel.dimmer:SetAllPoints(clawPanel)
clawPanel.dimmer:SetTexture(0, 0, 0, 0.6)
clawPanel.dimmer:Hide()
local clawPreviousPanel = CreateFrame('Frame', 'clawPreviousPanel', UIParent)
clawPreviousPanel:SetPoint('BOTTOMRIGHT', clawPanel, 'BOTTOMLEFT', -10, -5)
clawPreviousPanel:SetFrameStrata('BACKGROUND')
clawPreviousPanel:SetSize(64, 64)
clawPreviousPanel:Hide()
clawPreviousPanel:RegisterForDrag('LeftButton')
clawPreviousPanel:SetScript('OnDragStart', clawPreviousPanel.StartMoving)
clawPreviousPanel:SetScript('OnDragStop', clawPreviousPanel.StopMovingOrSizing)
clawPreviousPanel:SetMovable(true)
clawPreviousPanel.icon = clawPreviousPanel:CreateTexture(nil, 'BACKGROUND')
clawPreviousPanel.icon:SetAllPoints(clawPreviousPanel)
clawPreviousPanel.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
clawPreviousPanel.border = clawPreviousPanel:CreateTexture(nil, 'BORDER')
clawPreviousPanel.border:SetAllPoints(clawPreviousPanel)
clawPreviousPanel.border:SetTexture('Interface\\AddOns\\Claw\\border.blp')
local clawCooldownPanel = CreateFrame('Frame', 'clawCooldownPanel', UIParent)
clawCooldownPanel:SetPoint('BOTTOMLEFT', clawPanel, 'BOTTOMRIGHT', 10, -5)
clawCooldownPanel:SetSize(64, 64)
clawCooldownPanel:SetFrameStrata('BACKGROUND')
clawCooldownPanel:Hide()
clawCooldownPanel:RegisterForDrag('LeftButton')
clawCooldownPanel:SetScript('OnDragStart', clawCooldownPanel.StartMoving)
clawCooldownPanel:SetScript('OnDragStop', clawCooldownPanel.StopMovingOrSizing)
clawCooldownPanel:SetMovable(true)
clawCooldownPanel.icon = clawCooldownPanel:CreateTexture(nil, 'BACKGROUND')
clawCooldownPanel.icon:SetAllPoints(clawCooldownPanel)
clawCooldownPanel.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
clawCooldownPanel.border = clawCooldownPanel:CreateTexture(nil, 'BORDER')
clawCooldownPanel.border:SetAllPoints(clawCooldownPanel)
clawCooldownPanel.border:SetTexture('Interface\\AddOns\\Claw\\border.blp')

local function GetGCD()
	local startTime, duration = GetSpellCooldown(768)
	return max(duration - (currentTime - startTime) - Claw.latency, 0)
end

local Ability = {}
Ability.__index = Ability

function Ability.add(spellId, buff, playerCast, spellId2)
	local name, _, icon = GetSpellInfo(spellId)
	local ability = {
		spellId = spellId,
		spellId2 = spellId2 or 0,
		name = name,
		icon = icon,
		auraTarget = buff and 'player' or 'target',
		auraFilter = (buff and 'HELPFUL' or 'HARMFUL') .. (playerCast and '|PLAYER' or ''),
	}
	setmetatable(ability, Ability)
	return ability
end

function Ability:ready(seconds)
	return self:cooldown() <= max(seconds or 0, GetGCD())
end

function Ability:remains()
	local _, id, expires
	for i = 1, 40 do
		_, _, _, _, _, _, expires, _, _, _, id = UnitAura(self.auraTarget, i, self.auraFilter)
		if id == self.spellId or id == self.spellId2 then
			return max(expires - currentTime - Claw.latency, 0)
		end
	end
	return 0
end

function Ability:up()
	local _, id, duration, expires
	for i = 1, 40 do
		_, _, _, _, _, duration, expires, _, _, _, id = UnitAura(self.auraTarget, i, self.auraFilter)
		if id == self.spellId or id == self.spellId2 then
			return duration == 0 or expires - currentTime - Claw.latency > 0
		end
	end
end

function Ability:down()
	return not self:up()
end

function Ability:cooldown()
	local startTime, duration = GetSpellCooldown(self.spellId)
	return max(duration - (currentTime - startTime) - Claw.latency, 0)
end

function Ability:stack()
	local _, id, count
	for i = 1, 40 do
		_, _, _, count, _, _, _, _, _, _, id = UnitAura(self.auraTarget, i, self.auraFilter)
		if id == self.spellId then
			return count
		end
	end
	return 0
end

function Ability:known()
	return IsPlayerSpell(self.spellId) ~= false
end

function Ability:checkButtonAndGlow(button, on)
	if button and button:IsVisible() and button.icon:GetTexture() == self.icon then
		if on then
			ActionButton_ShowOverlayGlow(button)
		else
			ActionButton_HideOverlayGlow(button)
		end
	end
end

-- Druid abilities
local FaerieFire = Ability.add(770, false, false)
local ForceOfNature = Ability.add(102703, false, true)
local HealingTouch = Ability.add(5185, true, true)
local MarkOfTheWild = Ability.add(1126, true)
local OmenOfClarity = Ability.add(135700, true, true)
-- Cat abilities
local Berserk = Ability.add(106951, true, true)
local CatForm = Ability.add(768, true, true)
local Bloodtalons = Ability.add(155672, true, true)
local FerociousBite = Ability.add(22568, false, true)
local ImprovedRake = Ability.add(157276, false, true)
local IncarnationFeral = Ability.add(102543, true, true)
local PredatorySwiftness = Ability.add(69369, true, true)
local Prowl = Ability.add(5215, true, true)
local Rake = Ability.add(1822, false, true, 155722)
local Ravage = Ability.add(6785, false, true)
local Rip = Ability.add(1079, false, true)
local SavageRoar = Ability.add(52610, true, true, 174544)
local Shred = Ability.add(5221, false, true)
local Thrash = Ability.add(106830, false, true)
local TigersFury = Ability.add(5217, true, true)
-- Bear abilities
local BearForm = Ability.add(5487, true, true)
local BerserkBear = Ability.add(50334, true, true)
local Lacerate = Ability.add(33745, false, true)
local Mangle = Ability.add(33917, false, false)
local Maul = Ability.add(6807, false, true)
local Pulverize = Ability.add(80313, true, true)
local ThrashBear = Ability.add(77758, false, true)
-- Potions
local VirmensBite = Ability.add(105697, true, true)
-- Trinkets
local TickingEbonDetonator = Ability.add(146310, true, true)
-- Overpowered Racials
local Shadowmeld = Ability.add(58984, true, true)
-- Non-existent spell
local NoSpell = Ability.add(0)

local abilityMain, lastAbilityMain, abilityPrevious, abilityCD = NoSpell, NoSpell, NoSpell, NoSpell

Rake.targets, Rip.targets = {}, {}

local function ComboPoints()
	return GetComboPoints('player')
end

local EnergyRegen = GetPowerRegen

local function EnergyMax()
	return UnitPowerMax('player', 3)
end

local function Energy()
	return CatForm:up() and min(UnitPower('player', 3) + (EnergyRegen() * Claw.latency), EnergyMax()) or EnergyMax()
end

local function Mana()
	return UnitPower('player', 0)
end

local function EnergyTimeToMax()
	return (EnergyMax() - Energy()) / EnergyRegen()
end

local InCombat = InCombatLockdown

local function Rage()
	return BearForm:up() and UnitPower('player') or 0
end

local Target = {
	boss = false,
	guid = 0,
	healthArray = {},
	hostile = false
}

Target.healthPercentage = function()
	return UnitHealth('target') / UnitHealthMax('target') * 100
end

Target.timeToDie = function()
	local ttd = UnitHealth('target') / ((Target.healthArray[1] - Target.healthArray[#Target.healthArray]) / 3) - Claw.latency
	return ttd >= 0 and ttd or 600
end

function ForceOfNature:charges()
	return (GetSpellCharges(self.spellId))
end

local function BleedMultiplier()
	local multiplier = 1 + GetMasteryEffect() / 100
	if SavageRoar:up() then
		multiplier = multiplier * 1.4
	end
	if TigersFury:up() then
		multiplier = multiplier * 1.15
	end
	if Bloodtalons:up() then
		multiplier = multiplier * 1.3
	end
	return multiplier
end

function Rake:multiplier()
	return self:up() and self.targets[Target.guid] or 0
end

function Rake.newMultiplier()
	if ImprovedRake:known() and (Prowl:up() or Shadowmeld:up()) then
		return BleedMultiplier() + 1
	else
		return BleedMultiplier()
	end
end

function Rip:multiplier()
	return self:up() and self.targets[Target.guid] or 0
end

function Rip.newMultiplier()
	return BleedMultiplier()
end

function VirmensBite:cooldown()
	local startTime, duration = GetItemCooldown(76089)
	return max(duration - (currentTime - startTime) - Claw.latency, 0)
end

local function StatsBuffActive()
	local _, id, duration, expires
	for i = 1, 40 do
		_, _, _, _, _, duration, expires, _, _, _, id = UnitBuff('player', i)
		if id == 1126 or id == 20217 or id == 90363 or id == 117666 or id == 159988 or id == 160017 or id == 160077 then
			return duration == 0 or expires - currentTime > 10
		end
	end
end

local function DetermineAbilityCat() -- Adapted and modified for actual gameplay from SimulationCraft (Druid_Feral_T16M.simc, git commmit e535575f91cd, October 14, 2014), includes some enhancements by Defeat @ US-Arthas
	if Claw.cooldown and (not Claw.boss_only or Target.boss) then
		if ForceOfNature:known() and ForceOfNature:ready() and Prowl:down() and (ForceOfNature:charges() == 3 or (TickingEbonDetonator.equipped and TickingEbonDetonator:up())--[[add trinket buff up check]] or Target.timeToDie() < 20) then
			abilityCD = ForceOfNature
		elseif IncarnationFeral:known() and IncarnationFeral:ready() and TigersFury:ready() then
			abilityCD = IncarnationFeral
		elseif TigersFury:ready() and EnergyMax() - Energy() >= (OmenOfClarity:up() and 80 or 60) then
			abilityCD = Claw.tigersfury_berserk and Berserk:ready() and Berserk or TigersFury
		elseif Berserk:ready() and TigersFury:up() then
			abilityCD = Berserk
		elseif ImprovedRake:known() and Shadowmeld:known() and Shadowmeld:ready() and (Bloodtalons:up() or not Bloodtalons:known()) and Rake:remains() < 4.5 then
			abilityCD = Shadowmeld
		elseif Claw.pot and VirmensBite:ready() and Target.boss and ((Berserk:up() and Target.healthPercentage() < 25) or Target.timeToDie() < 40) then
			abilityCD = VirmensBite
		else
			abilityCD = NoSpell
		end
	else
		abilityCD = NoSpell
	end
	if Claw.mark_of_the_wild and not StatsBuffActive() then
		return MarkOfTheWild
	elseif not InCombat() then
		if Bloodtalons:known() and Bloodtalons:down() then
			return HealingTouch
		elseif CatForm:down() then
			return CatForm
		elseif Claw.prowl and Prowl:down() then
			return Prowl
		end
	end
	if Prowl:up() or Shadowmeld:up() then
		return Rake
	elseif ComboPoints() ~= 0 and Rip:up() and Rip:remains() < 3 and Target.healthPercentage() < 25 then
		return FerociousBite
	elseif Bloodtalons:known() and PredatorySwiftness:up() and (ComboPoints() >= 4 or PredatorySwiftness:remains() < 1.5) then
		return HealingTouch
	elseif SavageRoar:remains() < 3 and ComboPoints() >= 1 then
		return SavageRoar
	-- actions+=/thrash_cat,if=buff.omen_of_clarity.react&remains<=duration*0.3&active_enemies>1
	elseif ComboPoints() == 5 then
		if Target.healthPercentage() < 25 and Rip:up() and Energy() >= 50 then
			return FerociousBite
		elseif Rip:remains() < 3 or (Rip:remains() < 7.2 and Rip.newMultiplier() > Rip:multiplier()) then
			return Rip
		elseif EnergyTimeToMax() <= 1 or Berserk:up() or TigersFury:ready(3) then
			return SavageRoar:remains() < 12.6 and SavageRoar or FerociousBite
		end
	else
		if Rake:remains() < 3 or (Rake:remains() < 4.5 and Rake.newMultiplier() > Rake:multiplier()) then
			return Rake
		-- actions+=/thrash_cat,if=remains<=duration*0.3&active_enemies>1
		elseif Rake.newMultiplier() > Rake:multiplier() then
			return Rake
		-- actions+=/swipe,if=combo_points<5&active_enemies>=3
		else
			return Shred
		end
	end
	return NoSpell
end

local function DetermineAbilityBear() -- By myself, no simulation testing, max DPS scenario
	if Maul:ready() and Rage() >= 60 then
		abilityCD = Maul
	else
		abilityCD = NoSpell
	end
	if Claw.mark_of_the_wild and not StatsBuffActive() then
		return MarkOfTheWild
	elseif not InCombat() and BearForm:down() then
		return BearForm
	elseif Mangle:ready(0.4) then
		return Mangle
	elseif ThrashBear:ready(0.4) and ThrashBear:remains() < 1 then
		return ThrashBear
	elseif Lacerate:ready(0.4) and Lacerate:stack() ~= 3 then
		return Lacerate
	elseif Pulverize:known() and Lacerate:stack() == 3 then
		return Pulverize
	elseif Lacerate:ready(0.4) then
		return Lacerate
	elseif FaerieFire:ready(0.4) then
		return FaerieFire
	else
		return NoSpell
	end
end

local function DetermineAbility()
	if Claw.cat and BearForm:down() and GetSpecialization() == 2 then
		return DetermineAbilityCat()
	elseif Claw.bear and CatForm:down() and GetSpecialization() == 3 then
		return DetermineAbilityBear()
	else
		clawPreviousPanel:Hide()
		abilityCD = NoSpell
		return NoSpell
	end
end

local function DenyOverlayGlow(actionButton)
	if Claw.glow then
		actionButton.overlay:Hide()
	end
end

hooksecurefunc('ActionButton_ShowOverlayGlow', DenyOverlayGlow) -- Disable Blizzard's built-in action button glowing

local function StoreActionButtons()
	actionButtons = {}
	if Bartender4 then
		for i = 1, 120 do
			actionButtons[#actionButtons + 1] = _G['BT4Button' .. i]
		end
	elseif ElvUI then
		for b = 1, 6 do
			for i = 1, 12 do
				actionButtons[#actionButtons + 1] = _G['ElvUI_Bar' .. b .. 'Button' .. i]
			end
		end
	else
		for i = 1, 12 do
			actionButtons[#actionButtons + 1] = _G['ActionButton' .. i]
			actionButtons[#actionButtons + 1] = _G['MultiBarLeftButton' .. i]
			actionButtons[#actionButtons + 1] = _G['MultiBarRightButton' .. i]
			actionButtons[#actionButtons + 1] = _G['MultiBarBottomLeftButton' .. i]
			actionButtons[#actionButtons + 1] = _G['MultiBarBottomRightButton' .. i]
		end
		if Dominos then
			for i = 1, 60 do
				actionButtons[#actionButtons + 1] = _G['DominosActionButton' .. i]
			end
		end
	end
end

local function AddGlowToButtons()
	local button, w, h
	for i = 1, #actionButtons do
		button = actionButtons[i]
		w, h = button:GetSize()
		button.clawGlow = ActionButton_GetOverlayGlow()
		button.clawGlow:Hide()
		button.clawGlow:SetParent(button)
		button.clawGlow:ClearAllPoints()
		button.clawGlow:SetSize(w * 1.4, h * 1.4)
		button.clawGlow:SetPoint('TOPLEFT', button, 'TOPLEFT', -w * 0.2, h * 0.2)
		button.clawGlow:SetPoint('BOTTOMRIGHT', button, 'BOTTOMRIGHT', w * 0.2, -h * 0.2)
	end
end

local function UpdateButtonGlow()
	local button, icon
	for i = 1, #actionButtons do
		button = actionButtons[i]
		if button.clawGlow then
			icon = button.icon:GetTexture()
			if icon and (icon == abilityMain.icon or icon == abilityCD.icon) and button.icon:IsVisible() then
				if not button.clawGlow:IsVisible() then
					button.clawGlow.animIn:Play()
				end
			else
				button.clawGlow.animIn:Stop()
				button.clawGlow:Hide()
			end
		end
	end
end

function events:ACTIONBAR_SLOT_CHANGED()
	UpdateButtonGlow()
end

function events:PLAYER_LOGIN()
	me = UnitGUID('player')
	StoreActionButtons()
	AddGlowToButtons()
end

function events:PLAYER_TARGET_CHANGED()
	local previouslyHostile = Target.hostile
	Target.hostile = UnitCanAttack('player', 'target') and not UnitInVehicle('player')
	if Target.hostile then
		Target.guid = UnitGUID('target')
		Target.level = UnitLevel('target')
		Target.boss = Target.level == -1 or (Target.level >= UnitLevel('player') + 2 and not UnitInRaid('player'))
		for i = 1, #Target.healthArray do
			Target.healthArray[i] = UnitHealth('target')
		end
		abilityTimer = Claw.frequency
		clawPanel:Show()
	elseif previouslyHostile then
		abilityMain = NoSpell
		abilityCD = NoSpell
		if Claw.glow then
			UpdateButtonGlow()
		end
		clawPanel:Hide()
		clawPreviousPanel:Hide()
		clawCooldownPanel:Hide()
	end
end

function events:UNIT_FACTION(unitID)
	if unitID == 'target' then
		events:PLAYER_TARGET_CHANGED()
	end
end

local function Equipped(slot, item)
	return (GetInventoryItemLink('player', slot) or ''):match('%[([^%]]*)%]') == item and 1 or 0
end

function events:PLAYER_EQUIPMENT_CHANGED()
	TickingEbonDetonator.equipped = Equipped(13, 'Ticking Ebon Detonator') + Equipped(14, 'Ticking Ebon Detonator') >= 1
end

function events:PLAYER_REGEN_ENABLED()
	Rake.targets, Rip.targets = {}, {}
end

function events:ZONE_CHANGED_NEW_AREA()
	events:PLAYER_EQUIPMENT_CHANGED()
	clawPanel:UnregisterEvent('ZONE_CHANGED_NEW_AREA')
end

function events:ADDON_LOADED(name)
	if name == 'Claw' then
		if not Claw.frequency then
			print('It looks like this is your first time running Claw, why don\'t you take some time to familiarize yourself with the commands?')
			print('Type |cFFFFD000/claw|r for a list of commands.')
		end
		InitializeVariables()
		clawPanel:SetScale(Claw.scales.main)
		clawPanel:EnableMouse(not Claw.locked)
		clawPreviousPanel:SetScale(Claw.scales.previous)
		clawPreviousPanel:EnableMouse(not Claw.locked)
		clawCooldownPanel:SetScale(Claw.scales.cooldown)
		clawCooldownPanel:EnableMouse(not Claw.locked)
		for i = 1, floor(3 / Claw.frequency) do
			Target.healthArray[i] = 0
		end
	end
end

function events:COMBAT_LOG_EVENT_UNFILTERED(self, eventType, hideCaster, srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, spellId)
	if srcGUID == me then
		if eventType == 'SPELL_MISSED' and Claw.previous and Claw.miss_effect and spellId == abilityPrevious.spellId then
			clawPreviousPanel.border:SetTexture('Interface\\AddOns\\Claw\\misseffect.blp')
		elseif eventType == 'SPELL_CAST_SUCCESS' then
			if Claw.previous and spellId == abilityMain.spellId then
				abilityPrevious = abilityMain
				clawPreviousPanel.border:SetTexture('Interface\\AddOns\\Claw\\border.blp')
				clawPreviousPanel.icon:SetTexture(abilityPrevious.icon)
				clawPreviousPanel:Show()
			end
			if spellId == Rake.spellId then
				Rake.targets[dstGUID] = Rake.newMultiplier()
			elseif spellId == Rip.spellId then
				Rip.targets[dstGUID] = Rip.newMultiplier()
			end
		end
	end
end

clawPanel:SetScript('OnUpdate', function(self, elapsed)
	abilityTimer = abilityTimer + elapsed
	if abilityTimer >= Claw.frequency then
		Target.healthArray[#Target.healthArray + 1] = UnitHealth('target')
		table.remove(Target.healthArray, 1)
		lastAbilityMain = abilityMain
		lastAbilityCD = abilityCD
		currentTime = GetTime()
		abilityMain = DetermineAbility()
		if abilityMain ~= lastAbilityMain then
			if abilityMain.icon then
				clawPanel.icon:SetTexture(abilityMain.icon)
				clawPanel.icon:Show()
				clawPanel.border:Show()
			else
				clawPanel.icon:Hide()
				clawPanel.border:Hide()
			end
		end
		if abilityCD ~= lastAbilityCD then
			if abilityCD.icon then
				clawCooldownPanel.icon:SetTexture(abilityCD.icon)
				clawCooldownPanel:Show()
			else
				clawCooldownPanel:Hide()
			end
		end
		if Claw.glow and (abilityMain ~= lastAbilityMain or abilityCD ~= lastAbilityCD) then
			UpdateButtonGlow()
		end
		if Claw.gcd then
			local gcdStart, gcdDuration = GetSpellCooldown(768)
			if gcdStart == 0 then
				clawPanel.gcd:Hide()
			else
				clawPanel.gcd:SetCooldown(gcdStart, gcdDuration)
				clawPanel.gcd:Show()
			end
		end
		if Claw.dimmer then
			if abilityMain == NoSpell or IsUsableSpell(abilityMain.spellId) then
				clawPanel.dimmer:Hide()
			else
				clawPanel.dimmer:Show()
			end
		end
		abilityTimer = 0
	end
end)

clawPanel:SetScript('OnEvent', function(self, event, ...) events[event](self, ...) end)
for event in pairs(events) do
	clawPanel:RegisterEvent(event)
end

function SlashCmdList.Claw(msg, editbox)
	msg = { strsplit(' ', strlower(msg)) }
	if msg[1] == 'locked' then
		if msg[2] then
			Claw.locked = msg[2] == 'on'
			clawPanel:EnableMouse(not Claw.locked and clawPanel.icon:IsVisible())
			clawPreviousPanel:EnableMouse(not Claw.locked)
			clawCooldownPanel:EnableMouse(not Claw.locked)
		end
		print('Claw - Locked: ' .. (Claw.locked and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'scale' then
		if msg[2] == 'prev' then
			if msg[3] then
				Claw.scales.previous = tonumber(msg[3]) or 0.66
				clawPreviousPanel:SetScale(Claw.scales.previous)
			end
			print('Claw - Previous ability icon scale set to: |cFFFFD000' .. Claw.scales.previous .. '|r times')
		elseif msg[2] == 'main' then
			if msg[3] then
				Claw.scales.main = tonumber(msg[3]) or 1
				clawPanel:SetScale(Claw.scales.main)
			end
			print('Claw - Main ability icon scale set to: |cFFFFD000' .. Claw.scales.main .. '|r times')
		elseif msg[2] == 'cd' then
			if msg[3] then
				Claw.scales.cooldown = tonumber(msg[3]) or 0.66
				clawCooldownPanel:SetScale(Claw.scales.cooldown)
			end
			print('Claw - Cooldown ability icon scale set to: |cFFFFD000' .. Claw.scales.cooldown .. '|r times')
		else
			print('Claw - Default icon scale options are |cFFFFD000prev 0.66|r, |cFFFFD000main 1|r, and |cFFFFD000cd 0.66|r')
		end
	elseif msg[1] == 'frequency' then
		Claw.frequency = tonumber(msg[2]) or 0.05
		print('Claw - Calculation frequency: Every |cFFFFD000' .. Claw.frequency .. '|r seconds')
	elseif msg[1] == 'latency' then
		Claw.latency = tonumber(msg[2]) or 0.2
		print('Claw - Player reaction time: |cFFFFD000' .. Claw.latency .. '|r seconds')
	elseif msg[1] == 'glow' then
		if msg[2] then
			Claw.glow = msg[2] == 'on'
			UpdateButtonGlow()
		end
		print('Claw - Glowing ability buttons: ' .. (Claw.glow and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'previous' then
		if msg[2] then Claw.previous = msg[2] == 'on' end
		print('Claw - Previous ability icon: ' .. (Claw.previous and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'cd' then
		if msg[2] then Claw.cooldown = msg[2] == 'on' end
		print('Claw - Cooldown ability icon: ' .. (Claw.cooldown and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'gcd' then
		if msg[2] then Claw.gcd = msg[2] == 'on' end
		if not Claw.gcd then
			clawPanel.gcd:Hide()
		end
		print('Claw - Global cooldown swipe: ' .. (Claw.gcd and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'dim' then
		if msg[2] then Claw.dimmer = msg[2] == 'on' end
		clawPanel.dimmer:Hide()
		print('Claw - Dim main ability icon when you don\'t have enough rage to use it: ' .. (Claw.dimmer and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'misseffect' then
		if msg[2] then Claw.miss_effect = msg[2] == 'on' end
		print('Claw - Red border around previous ability when it fails to hit: ' .. (Claw.miss_effect and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'bossonly' then
		if msg[2] then Claw.boss_only = msg[2] == 'on' end
		print('Claw - Only use cooldowns on bosses: ' .. (Claw.boss_only and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'motw' then
		if msg[2] then Claw.mark_of_the_wild = msg[2] == 'on' end
		print('Claw - Mark of the Wild: ' .. (Claw.mark_of_the_wild and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'bear' then
		if msg[2] then Claw.bear = msg[2] == 'on' end
		print('Claw - Show Claw UI in bear (guardian) specialization: ' .. (Claw.bear and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'cat' then
		if msg[2] then Claw.cat = msg[2] == 'on' end
		print('Claw - Show Claw UI in cat (feral) specialization: ' .. (Claw.cat and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'prowl' then
		if msg[2] then Claw.prowl = msg[2] == 'on' end
		print('Claw - Start combat with Prowl (cat only): ' .. (Claw.prowl and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'tfberserk' then
		if msg[2] then Claw.tigersfury_berserk = msg[2] == 'on' end
		print('Claw - Tiger\'s Fury is macro\'d to Berserk (cat only): ' .. (Claw.tigersfury_berserk and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'pot' then
		if msg[2] then Claw.pot = msg[2] == 'on' end
		print('Claw - show Virmen\'s Bite potions in cooldown UI (cat only): ' .. (Claw.pot and '|cFF00C000On' or '|cFFC00000Off'))
	elseif msg[1] == 'reset' then
		clawPanel:ClearAllPoints()
		clawPanel:SetPoint('CENTER', 0, -169)
		clawPreviousPanel:ClearAllPoints()
		clawPreviousPanel:SetPoint('BOTTOMRIGHT', clawPanel, 'BOTTOMLEFT', -10, -5)
		clawCooldownPanel:ClearAllPoints()
		clawCooldownPanel:SetPoint('BOTTOMLEFT', clawPanel, 'BOTTOMRIGHT', 10, -5)
		print('Claw - Position has been reset to default')
	else
		print('Claw (version: |cFFFFD000' .. GetAddOnMetadata('Claw', 'Version') .. '|r) - Commands:')
		print('  /claw locked |cFF00C000on|r/|cFFC00000off|r - lock the Claw UI so that it can\'t be moved')
		print('  /claw scale |cFFFFD000prev|r/|cFFFFD000main|r/|cFFFFD000cd|r |cFFFFD000[number]|r - adjust the scale of the Claw UI icons')
		print('  /claw frequency |cFFFFD000[number]|r - set the calculation frequency (default is every 0.05 seconds)')
		print('  /claw latency |cFFFFD000[number]|r - player\'s reaction time (default is 0.2 seconds)')
		print('  /claw glow |cFF00C000on|r/|cFFC00000off|r - glowing ability buttons on action bars')
		print('  /claw previous |cFF00C000on|r/|cFFC00000off|r - previous ability icon')
		print('  /claw cd |cFF00C000on|r/|cFFC00000off|r - cooldown ability icon (turn off for manual cooldown management)')
		print('  /claw gcd |cFF00C000on|r/|cFFC00000off|r - show global cooldown swipe on main ability icon')
		print('  /claw dim |cFF00C000on|r/|cFFC00000off|r - dim main ability icon when you don\'t have enough mana/energy')
		print('  /claw misseffect |cFF00C000on|r/|cFFC00000off|r - red border around previous ability when it fails to hit')
		print('  /claw bossonly |cFF00C000on|r/|cFFC00000off|r - only use cooldowns on bosses')
		print('  /claw motw |cFF00C000on|r/|cFFC00000off|r - use Mark of the Wild')
		print('  /claw bear |cFF00C000on|r/|cFFC00000off|r - show Claw UI in bear (guardian) specialization')
		print('  /claw cat |cFF00C000on|r/|cFFC00000off|r - show Claw UI in cat (feral) specialization')
		print('  /claw prowl |cFF00C000on|r/|cFFC00000off|r - start combat with Prowl (cat only)')
		print('  /claw tfberserk |cFF00C000on|r/|cFFC00000off|r - Tiger\'s Fury is macro\'d to Berserk (cat only)')
		print('  /claw pot |cFF00C000on|r/|cFFC00000off|r - show Virmen\'s Bite potions in cooldown UI (cat only)')
		print('  /claw |cFFFFD000reset|r - reset the location of the Claw UI to default')
		if Basic_Resources then
			print('For Basic Resources commands, please type |cFFFFD000/bres|r.')
		end
	end
end